<?php
/*
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
class Montonio_Gateway_Block_Createdraft extends Mage_Payment_Block_Form
{

    protected function _construct()
    {
        $this->redirectToMontonio();
    }

    public function getSession()
    {
        return Mage::getSingleton('checkout/session');
    }

    public function getBaseSecureUrl()
    {
        return Mage::app()->getStore(Mage::app()->getStore()->getStoreId())->getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK, true);
    }

    public function getApplicationUrl()
    {
        $uri = Mage::helper('montonio')->isSandbox() ? 'https://sandbox-application.montonio.com' : 'https://application.montonio.com'; 
        return $uri;
    }

    public function getRedirectUrl()
    {

        // get order
        $order = Mage::getModel('sales/order')->load(Mage::getSingleton('checkout/session')->getLastOrderId());
        if (!$order) {
            throw new Exception('2 An error occurred while initiating Montonio payment.');
        }

        // check method code
        $methodCode = $order->getPayment()->getMethodInstance()->getCode();
        if ($methodCode !== Montonio_Gateway_Model_PaymentMethod::METHOD_CODE) {
            throw new Exception('3 An error occurred while initiating Montonio payment.');
        }

        // Create application draft for Montonio API
        try {
            $draftData = array(
                "origin"                => "online",
                "merchant_reference"    => (string) $order->increment_id,
                "customer_email"        => (string) $order->customer_email,
                "customer_last_name"    => (string) $order->customer_lastname,
                "customer_first_name"   => (string) $order->customer_firstname,
                "callback_url"          => Mage::getUrl('montonio/redirect/return', array('_query' => 'reference=' . (string) $order->increment_id)),
                "products"              => array()
            );

            // add products to $draftData
            foreach ($order->getAllItems() as $key => $product) {
                $draftData['products'][] = array(
                    'product_name'  =>         $product->getName(),
                    'product_price' => (float) $product->getPriceInclTax(),
                    'quantity'      => (int)   $product->getQtyOrdered()
                );
            }
            
            // add shipping cost to $draftData
            $shippingCost = $order->getShippingAmount();
            if ($shippingCost > 0) {
                $draftData['products'][] = array(
                    'quantity'      => 1,
                    'product_name'  => 'SHIPPING',
                    'product_price' => (float) $shippingCost
                );
            }
        } catch (Exception $e) {
            // error in composing draft data
            throw new Exception('4 An error occurred while initiating Montonio payment.');
        }

        // remove product variations that cost 0.00 from draft data
        $draftData['products'] = array_values(array_filter($draftData['products'], 'self::removeProductVariation'));

        // init Montonio API
        require_once(Mage::getBaseDir('lib') . DS . 'Montonio' . DS . 'MontonioAPI.php');
        $api = new MontonioAPI(
            Mage::helper('montonio')->getAccessKey(),
            Mage::helper('montonio')->getSecretKey(),
            Mage::helper('montonio')->getEnvironment()
        );

        // submit application draft and get token
        $response = $api->post_montonio_application_draft(json_encode($draftData));

        if ($response['status'] !== 'SUCCESS') {
            throw new Exception('5 An error occurred while initiating Montonio payment.');
        }

        return $this->getApplicationUrl() . '?access_token=' . $response['data']->access_token;
     }

    public function redirectToMontonio()
    {
        try {
            Mage::app()->getResponse()->setRedirect($this->getRedirectUrl());
        } catch (Exception $e) {
            Mage::getSingleton('core/session')->addError($e->getMessage());
        }
    }

    protected static function removeProductVariation($e)
    {
        return $e['product_price'] > 0;
    }
}
