<?php
/**
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */

//require JWT lib
require_once Mage::getBaseDir('lib') . DS . 'Montonio' . DS . 'JWT.php';

class Montonio_Gateway_Model_Method_Payments extends Mage_Payment_Model_Method_Abstract
{

    const METHOD_CODE = 'montonio_payments';
    const METHOD_ALL_COUNTRIES = 'ALL';
    const GATEWAY_METHOD_CODE = 'paymentInitiation';
    const GATEWAY_LOCALES = [
        'en_US' => 'en_US',
        'et_EE' => 'et',
        'lt_LT' => 'lt',
        'lv_LV' => 'lv',
        'ru_RU' => 'ru',
        'de_DE' => 'de',
        'en_GB' => 'en_US',
        'fi_FI' => 'fi',
        'pl_PL' => 'pl',
    ];
    const GATEWAY_PREF_COUNTRIES = [
        'EE' => 'EE',
        'LV' => 'LV',
        'LT' => 'LT',
        'FI' => 'FI',
        'PL' => 'PL',
        'DE' => 'DE'
    ];

    protected $_formBlockType               = 'montonio/payments_form';
    protected $_infoBlockType               = 'montonio/payments_info';
    protected $_code                        = self::METHOD_CODE;
    protected $_allowCurrencyCode           = ['EUR'];
    protected $_canManageRecurringProfiles  = false;
    protected $_canUseInternal              = false;

    /**
     * @inheritdoc
     */
    public function getCode()
    {
        $virtualCode = $this->getData('code');
        if ($virtualCode && strstr($virtualCode, self::METHOD_CODE.'_')) {
            return $virtualCode;
        } else {
            return parent::getCode();
        }
    }

    /**
     * @inheritdoc
     */
    public function getConfigData($field, $storeId = null)
    {
        if (null === $storeId) {
            $storeId = $this->getStore();
        }

        //virtual code is sometimes needed but not on config calls
        $code = $this->getCode();

        if (strstr($code, 'montonio_payments_')) {
            $subCode     = str_replace('montonio_payments_', '', $code);
            $methods = $this->getAvailableMethods();
            if ($field == 'country') {
                if (isset($methods[$subCode])) {
                    return $methods[$subCode]['method_country'];
                }
            }
            if ($field == 'title') {
                if (isset($methods[$subCode])) {
                    return $methods[$subCode]['method_title'];
                }
            }
            if ($field == 'description') {
                if (isset($methods[$subCode])) {
                    return $methods[$subCode]['method_desc'];
                }
            }
            if ($field == 'logo') {
                if (isset($methods[$subCode])) {
                    return $methods[$subCode]['logo_url'];
                }
            }
            if ($field == 'sort_order') {
                if (isset($methods[$subCode])) {
                    return $methods[$subCode]['method_sort'];
                }
            }
        }

        $path = 'payment/'. $this->getCode().'/'.$field;
        return Mage::getStoreConfig($path, $storeId);
    }

    /**
     * @inheritdoc
     */
    public function getOrderPlaceRedirectUrl()
    {
        $url = Mage::getUrl('checkout/cart');

        try {
            $info               = $this->getInfoInstance();
            $data               = $info->getAdditionalInformation();
            //add accessKey, exp
            $storeId            = $info->getQuote()->getStoreId();
            $environment        = $this->getConfigData('environment', $storeId);
            $time               = Mage::getModel('core/date')->timestamp(time());
            $data[$this->getCode() . '_data']['exp']        = $time + (60 * 10);
            $data[$this->getCode() . '_data']['accessKey']  = $this->getConfigData($environment.'_public_key', $storeId);
            //get token
            $privateKey         = $this->getConfigData($environment.'_private_key', $storeId);
            $token              = $this->generateSignature($data[$this->getCode() . '_data'], $privateKey);
            //post preflight
            $url = $this->getConfigData($environment.'_application_url', $storeId) . '/orders';
            $responseUrl = $this->postPayload($token, $url);

            if(!isset($responseUrl['paymentUrl'])) {
                Mage::logException(print_r($responseUrl, true));
                throw new Mage_Core_Exception('Something wen\'t wrong while placing the payment');

            }
        } catch (Exception $e) {
            Mage::logException($e);
            throw new Mage_Core_Exception($e->getMessage());
        }

        return $responseUrl['paymentUrl'];
    }


    /**
     * Preflight request to get redirection url form GW
     *
     * @param $token
     * @param $url
     * @return array
     */
    protected function postPayload($token, $url) {

        $client = new Varien_Http_Client();
        $client->setUri((string)$url);
        $client->setHeaders('Content-Type','application/json');
        $client->setConfig([
            'maxredirects' => 0,
            'timeout' => 30
        ]);

        $client->setRawData(
            json_encode([
                'body' => $token
            ])
        );

        $result = $client->request(Varien_Http_Client::POST);

        return json_decode($result->getBody(), true);
    }

    /**
     *
     * @param  $order
     * @param  $payment
     * @return array
     */
    public function prepPayload($order)
    {

        $storeId        = $order->getStoreId();
        $returnUrl      = $this->getConfigData('return_url', $storeId);
        $hookUrl        = $this->getConfigData('return_hook_url', $storeId);
        $storeLocale    = Mage::app()->getLocale()->getLocaleCode();
        //montonio only supports following locales

        $locales = self::GATEWAY_LOCALES;
        $localeCode = (isset($locales[$storeLocale])) ? $locales[$storeLocale] : $locales['en_US'];

        $data = [
            'merchantReference' => (string)$order->getIncrementId(),
            'returnUrl'         => (string)Mage::getUrl($returnUrl),
            'notificationUrl'   => (string)Mage::getUrl($hookUrl),
            'currency'          => (string)$order->getOrderCurrency()->toString(),
            'grandTotal'        => (float)sprintf("%.2f", $order->getGrandTotal()),
            'locale'            => $localeCode,
        ];

        $data['billingAddress'] = $this->getAddressPayload($order->getBillingAddress(), $order->getCustomerEmail());

        if($order->getQuote()->getIsVirtual()) {
            $shippingAddress = $order->getBillingAddress();
        } else {
            $shippingAddress = $order->getShippingAddress();
        }
        $data['shippingAddress'] = $this->getAddressPayload($shippingAddress, $order->getCustomerEmail());

        foreach ($order->getAllItems() as $item) {
            if(!$item->isDeleted() && !$item->getParentItem()) {
                $data['lineItems'][] = [
                    'name'       => (string)Mage::helper('core')->escapeHtml($item->getName()),
                    'quantity'   => (float)$item->getQtyOrdered(),
                    'finalPrice' => (float)($item->getPriceInclTax() - $item->getDiscountAmount()),
                ];
            }
        }

        $data['payment'] = [
            'method'        => self::GATEWAY_METHOD_CODE,
            'methodDisplay' => (string)Mage::helper('core')->escapeHtml($this->getConfigData('title')),
            'amount'        => $data['grandTotal'], // Yes, this is the same as order->grandTotal.
            'currency'      => (string)$data['currency'], // This must match the currency of the order.
            'methodOptions' => [
                //'paymentReference'   => '1010109',
                'paymentDescription' => (string)Mage::helper('montonio')
                    ->__('Payment for order "%s"', $data['merchantReference']),
                'preferredLocale'    => $localeCode
            ]
        ];

        $prefCountry = Mage::app()->getRequest()->getParam('montonio-payments-preferred-country');
        if(!empty($prefCountry)) {
            $allowedCountries = self::GATEWAY_PREF_COUNTRIES;
            $prefCountry = (isset($allowedCountries[$prefCountry])) ?
                $allowedCountries[$prefCountry] : current(self::GATEWAY_PREF_COUNTRIES);
            $data['payment']
                ['methodOptions']
                ['preferredCountry'] = (string)Mage::helper('core')->escapeHtml($prefCountry);
        }

        $prefBank = Mage::app()->getRequest()->getParam('montonio-payments-preselected-bank');
        if(!empty($prefBank)) {
            $data['payment']
                ['methodOptions']
                ['preferredProvider'] = (string)Mage::helper('core')->escapeHtml($prefBank);
        }

        return $data;
    }

    /**
     * Get payload address structured data from address object
     *
     * @param  $address
     * @return array
     */
    protected function getAddressPayload($address, $email)
    {
        $street = $address->getStreet();
        $street = (is_array($street)) ? implode(', ', $street) : $street;
        $data = [
            'firstName' => (string)$address->getFirstname(),
            'lastName' => (string)$address->getLastname(),
            'email' => (string)$email,
            'addressLine1' => (string) $street,
            'locality' => (string)$address->getCity(),
            'region' => (string)$address->getRegion(),
            'postalCode' => (string)$address->getPostcode(),
            'phoneNumber' => (string)$address->getPhonenumber(),
            'country' => (string)$address->getCountry()
        ];

        return array_filter($data);
    }

    /**
     *
     * @param mixed $data
     * @param string $key
     * @return string
     */
    public function generateSignature($data, $key, $jsonEncode = false)
    {
        if ($jsonEncode) {
            $data = json_encode($data);
        }
        return JWT::encode($data, $key);
    }

    /**
     *
     * @param string $data
     * @param string $key
     * @return array|string
     */
    public function decodeSignature($data, $key)
    {
        return JWT::decode($data, $key);
    }

    /**
     *
     * @param array $data
     * @param string $accesKey
     * @return boolean
     */
    public function validateSignedData($data, $accesKey)
    {
        $requiredKeys = explode(',',$this->getConfigData('validate_fields'));
        $time   = Mage::getModel('core/date')->timestamp(time());
        $data = (array)$data;

        foreach($requiredKeys as $k => $v) {
            if(!isset($data[$v])) {
                Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on missing field in message: "%s"', $v));
            }
        }
        if ($data['accessKey'] !== $accesKey) {
            Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on "%s" validation', 'access_key'));

        }
        if ($data['exp'] < $time) {
            Mage::throwException(Mage::helper('montonio')->__('Signature validation failed on "%s" validation', 'exp'));
        }

        return true;
    }

    /**
     *
     * @param string $token
     * @param string $storeId
     * @return array
     */
    public function validateIncomingData($token, $storeId = null)
    {

        $environment    = $this->getConfigData('environment');
        $privateKey     = $this->getConfigData($environment.'_private_key', $storeId);
        $accessKey      = $this->getConfigData($environment.'_public_key', $storeId);
        $data           = (array)$this->decodeSignature($token, $privateKey);
        try {
            $this->validateSignedData($data, $accessKey);
        } catch (\Exception $e) {
            Mage::logException($e);
            $data = [];
        }

        return $data;
    }

    /**
     * Handle order status on response data from gateway
     *
     * @param $data
     * @param $response
     * @return array
     */
    public function handleOrderStatus($data, $response)
    {

        $order  = Mage::getModel('sales/order')
                    ->loadByIncrementId($data['merchantReference']);
        $helper = Mage::helper('montonio');

        if (!isset($order['increment_id']) ||
            $order['increment_id'] !== $data['merchantReference'] ||
            $order['status'] !== 'pending'
        ) {
                $response['status'] = $order['status'];
                return $response;
        }

        $quote = Mage::getModel('sales/quote')
                         ->load($order['quote_id']);

        if (strtolower($data['paymentStatus']) === 'paid') {
            $newStatus = Mage_Sales_Model_Order::STATE_PROCESSING;
            $message = $helper->__('Payment accepted in payment gateway for "%s"', $data['grandTotal']);
            $quote->setIsActive(0);
        } elseif (strtolower($data['paymentStatus']) === 'pending') {
            $newStatus = Mage_Sales_Model_Order::STATE_PENDING_PAYMENT;
            $message = $helper->__('Payment status "pending", status will be updated with webhook request');
            $quote->setIsActive(0);
        } else {
            $newStatus = Mage_Sales_Model_Order::STATE_CANCELED;
            $message = $helper->__('Payment cancelled in payment gateway');
            $quote->setIsActive(1);
        }

        $transaction = Mage::getModel('core/resource_transaction');
        $transaction->addObject($quote);

        $order->setState($newStatus, true, $message, false);
        $order->setCanSendNewEmailFlag(true);

        $canCreateInvoice = $this->getConfigData('create_invoice', $order->getStoreId());

        //add invoice to order
        if (
            $order->getState() === Mage_Sales_Model_Order::STATE_PROCESSING &&
            $order->canInvoice() &&
            $canCreateInvoice
        ) {

            $invoice = Mage::getModel('sales/service_order', $order)->prepareInvoice();
            $invoice->setRequestedCaptureCase(Mage_Sales_Model_Order_Invoice::CAPTURE_OFFLINE);
            $invoice->register();
            $invoice->addComment($message,false,false);
            $invoice->getOrder()->setIsInProcess(true);

            $transaction->addObject($invoice);

            $order = $invoice->getOrder();

        }

        $transaction->addObject($order);
        $transaction->save();

        try {

            $canOrderEmail = $this->getConfigData('order_email_after', $order->getStoreId());

            if($order->getCanSendNewEmailFlag() && $canOrderEmail) {
                //for 1.9.* it turned to queue based method
                if(method_exists ($order, 'queueNewOrderEmail')) {
                    $order->queueNewOrderEmail();
                } else {
                    //for anything lower than 1.9 we need to use the old method
                    if(method_exists ($order, 'sendNewOrderEmail')) {
                        $order->sendNewOrderEmail();
                    }
                }
            }

        } catch (Exception $e) {
            Mage::logException($e);
        }

        try {

            $canInvoiceEmail = $this->getConfigData('invoice_email_after', $order->getStoreId());

            if($canCreateInvoice && is_object($invoice) && $canInvoiceEmail) {
                if(method_exists ($invoice, 'sendEmail')) {
                    $invoice->sendEmail();
                }
            }

        } catch (Exception $e) {
            Mage::logException($e);
        }

        $response = [
            'success' => true,
            'error' => false,
            'status' => $newStatus,
            'message' => $message
        ];

        if ($newStatus === Mage_Sales_Model_Order::STATE_PROCESSING) {
            $response['data'] = [
                'order_increment_id' => $order['increment_id'],
                'order_id' => $order['entity_id'],
                'quote_id' => $order['quote_id'],
            ];
        }

        return $response;
    }

    /**
     * get list of available Montonio methods from config
     *
     * @return array
     */
    public function getAvailableMethods()
    {
        $methods = [];

        $cacheId = $this->getCode(). '_' .  Mage::app()->getStore()->getId();
        $ttl = Mage::app()->loadCache($cacheId. '_ttl');
        $time = Mage::getModel('core/date')->timestamp(time());
        $extra = 24 * 60 * 60;

        if(!$ttl || $ttl < $time) {
            Mage::app()->saveCache($this->fetchAvailableMethodsFromApi(), $cacheId . '_available_methods', [Mage_Core_Model_Config::CACHE_TAG]);
            Mage::app()->saveCache($time+$extra, $cacheId . '_ttl', [Mage_Core_Model_Config::CACHE_TAG]);
        }

        $values = Mage::app()->loadCache($cacheId. '_available_methods');;

        try {
            $methods = json_decode($values, true)['paymentMethods']['paymentInitiation']['setup'];
        } catch (\Exception $e) {
            Mage::logException($e);
        }

        return $methods;
    }

    /**
     * Executes curl to get list of banks from gateway
     *
     * @throws \Exception
     * @return array
     */
    public function fetchAvailableMethodsFromApi()
    {

        $response = [];
        $environment    = $this->getConfigData('environment');

        $time           = Mage::getModel('core/date')->timestamp(time());
        $pubKey         = $this->getConfigData($environment.'_public_key');
        $privKey        = $this->getConfigData($environment.'_private_key');
        $apiUrl         = $this->getConfigData($environment.'_application_url');

        if (empty($privKey)) {
            return $response;
        }

        $apiEndpoint    = '/stores/setup';
        $transferObject = new Varien_Object();
        $payload = [
            'accessKey' => $pubKey,
            'iat'        => $time,
            'exp'        => $time + (60 * 60)
        ];

        try {
            $bearerJwtToken = JWT::encode($payload, $privKey,'HS256');
        } catch (Exception $e) {
            Mage::logException($e);
            throw new \Exception($e->getMessage());
        }

        $transferObject
            ->setUri($apiUrl.$apiEndpoint)
            ->setHeaders(
                [
                    'Authorization: Bearer ' . $bearerJwtToken
                ]
            )
            ->setMethod('GET');

        try {
            $http = new Varien_Http_Adapter_Curl();
            $config = [
                'timeout'       => 30,
                'verifypeer'    => 0,
                'verifyhost'    => 0,
                'header'        => true,
            ];
            $http->setConfig($config);
            $http->write(
                $transferObject->getMethod(),
                $transferObject->getUri(),
                '1.1',
                $transferObject->getHeaders()
            );
            $response = $http->read();
            $response =  Zend_Http_Response::extractBody($response);
            $http->close();

            if (isset($response['statusCode']) && $response['statusCode'] !== '200') {
                $message = Mage::helper('montonio')->
                    __('Fetching payment method list request status "%s".', $response['statusCode']);
                throw new \Exception($message);
            }

        } catch (\Exception $e) {
            Mage::logException($e);
            throw new \Exception($e->getMessage());
        }

        return $response;
    }

    /**
     *
     * @inheritdoc
     */
    public function canUseForCountry($countryCode) {

        $result = false;

        if(
            $this->getConfigData('country') !== '' &&
            (
                $countryCode == $this->getConfigData('country') ||
                $this->getConfigData('country') === self::METHOD_ALL_COUNTRIES ||
                $this->getConfigData('allowall') == 1
            )
        ) {
            $result =  true;
        } else if ($this->getConfigData('country') == '') {
            $result =  parent::canUseForCountry($countryCode);
        }

        return $result;
    }
}
