<?php
/**
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
class Montonio_Gateway_PaymentsController extends Mage_Core_Controller_Front_Action
{
    protected $_redirectBlockType = 'montonio/redirect';
    protected $_paymentInst = null;

    public function getCheckout()
    {
        return Mage::getSingleton('checkout/session');
    }

    public function indexAction()
    {

        $token = (string)$this->getRequest()->getParam('order-token');
        $session = $this->getCheckout();
        $response = [
            'success' => false,
            'error' => true,
            'status' => 'fail',
            'message' => Mage::helper('montonio')->__('Nothing to do here')
        ];

        if (!empty($token)) {
            $payment = Mage::getModel('montonio/method_payments');
            $data = $payment->validateIncomingData($token);
            if (!empty($data)) {
                $response = $payment->handleOrderStatus($data, $response);
            }
        }

        if ($response['success']) {
            if ($response['status'] === Mage_Sales_Model_Order::STATE_CANCELED) {
                $url = 'checkout/cart';
                $session->addSuccess($response['message']);
            }
            if ($response['status'] === Mage_Sales_Model_Order::STATE_PROCESSING) {
                $url = 'checkout/onepage/success';
                // add order/quote information to the session
                //for success page (if between return and redirect is a gap with new data)
                $session
                    ->setLastOrderId($response['data']['order_id'])
                    ->setLastRealOrderId($response['data']['order_increment_id'])
                    ->setLastQuoteId($response['data']['quote_id'])
                    ->setLastSuccessQuoteId($response['data']['quote_id']);
            }
        }
        if ($response['error']) {
            $url = 'checkout/cart';

            if ($response['status'] === Mage_Sales_Model_Order::STATE_PROCESSING) {
                $url = 'checkout/onepage/success';
            } else {
                $session->addError(Mage::helper('montonio')->__('Payment confirmation failed.'));
            }
        }
        $this->_redirect($url);
        return;
    }

    /**
     * This is called to redirect customer to Montonio
     *
     * @return void
     */
    public function hookAction()
    {
        $token = $this->getRequest()->getParam('order-token');
        $session = $this->getCheckout();
        $response = [
            'success' => false,
            'error' => true,
            'status' => 'fail',
            'message' => Mage::helper('montonio')->__('Nothing to do here')
        ];

        if (!empty($token)) {
            $payment = Mage::getModel('montonio/method_payments');
            $data = $payment->validateIncomingData($token);
            if (!empty($data)) {
                $response = $payment->handleOrderStatus($data, $response);
            }
        }
        if (isset($response['data'])) {
            unset($response['data']);
        }
        $this->_prepareDataJSON($response);
        return;
    }

    /**
     * Prepare JSON formatted data for response to client
     *
     * @param $response
     * @return Zend_Controller_Response_Abstract
     */
    protected function _prepareDataJSON($response)
    {
        $this->getResponse()->setHeader('Content-type', 'application/json', true);
        return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($response));
    }
}
