<?php
/**
 * Montonio_Gateway
 *
 * @category   montonio
 * @package    Montonio_Gateway
 * @copyright  Copyright (c) 2020 Montonio Finance OÜ  (https://montonio.com/)
 * @license    https://www.gnu.org/licenses/gpl-3.0.txt
 */
class Montonio_Gateway_RedirectController extends Mage_Core_Controller_Front_Action
{
    protected $_redirectBlockType = 'montonio/redirect';
    protected $_paymentInst = null;

    public function getCheckout()
    {
        return Mage::getSingleton('checkout/session');
    }

    /**
    * This is called after 'Place order' button clicked at checkout
    * 
    * @return string URL to redirect to 
    */
    public function redirectAction()
    {
        $session = $this->getCheckout();

        $order = Mage::getModel('sales/order');
        $order->loadByIncrementId($session->getLastRealOrderId());
        $order->save();

        $this->_redirect('montonio/redirect/create');
    }

    /**
     * This is called to redirect customer to Montonio
     *
     * @return void
     */
    public function createAction()
    {
        $this->loadLayout();
        $this->_initLayoutMessages('checkout/session');
        $this->getLayout()->getBlock('content')->append($this->getLayout()->createBlock('montonio/createdraft'));
        $this->renderLayout();
    }

    /**
     * Resolve callback from Montonio and redirect user
     *
     * @return void
     */
    public function returnAction()
    {
        $montonio = Mage::getModel('montonio/PaymentMethod');
        $result = $montonio->validateApplication($_REQUEST);

        try {
            if ($result === true) {
                $quote = Mage::getSingleton('checkout/cart')->getQuote();
                $quote->setIsActive(0);
                $quote->save();

                $this->_redirect('checkout/onepage/success');
            } else {
                $this->_redirect('checkout/onepage/failure');
            }
        } catch (Exception $e) {

        }
    }
}
